<?php

namespace Drupal\entity_fallback_value\PluginManager;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Class TreePluginManager.
 *
 * @package Drupal\entity_fallback_value\PluginManager
 */
class EntityFallbackValuePluginManager extends DefaultPluginManager {

  /**
   * Nom du service.
   *
   * @const string
   */
  const SERVICE_NAME = 'entity_fallback_value.plugin_manager';

  /**
   * Package name.
   *
   * @const string
   */
  const PACKAGE_NAME = 'entity_fallback_value';

  /**
   * Instances cache.
   *
   * @var \Drupal\entity_fallback_value\PluginManager\Plugin\EntityFallbackValuePluginInterface[]|null
   */
  protected $instances = NULL;

  /**
   * Singleton.
   *
   * @return static
   *   Le singleton.
   */
  public static function me() {
    return \Drupal::service(static::SERVICE_NAME);
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct(
      'Plugin/' . static::PACKAGE_NAME,
      $namespaces,
      $module_handler,
      'Drupal\\' . static::PACKAGE_NAME . '\PluginManager\Plugin\EntityFallbackValuePluginInterface',
      'Drupal\\' . static::PACKAGE_NAME . '\PluginManager\Annotation\EntityFallbackValuePluginAnnotation'
    );

    $hookName = static::PACKAGE_NAME . '_info';
    $this->alterInfo($hookName);
  }

  /**
   * Return the plugins that apply on $contentEntityBase.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $contentEntityBase
   *   The base entity.
   *
   * @return array
   *   The list of plugins.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getInstancesByContentEntity(ContentEntityInterface $contentEntityBase): array {
    $instances = [];
    foreach ($this->getInstances() as $instance) {
      if ($instance->applies($contentEntityBase)) {
        $instances[] = $instance;
      }
    }

    return $instances;
  }

  /**
   * Get the list of plugin instances.
   *
   * @return \Drupal\entity_fallback_value\PluginManager\Plugin\EntityFallbackValuePluginInterface[]|null
   *   The instances.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  protected function getInstances() {
    if (is_null($this->instances)) {
      foreach ($this->getDefinitions() as $definition) {
        $pluginID = $definition['id'];
        $instance = parent::createInstance($pluginID, $this->getAppliesOnArray($definition['applies_on']));
        $this->instances[$pluginID] = $instance;
      }
    }

    return $this->instances;
  }

  /**
   * Return an array of entity_type_id => [bundles].
   *
   * @param array $appliesOn
   *   THe definition values.
   *
   * @return array
   *   THe applies on array.
   */
  protected function getAppliesOnArray(array $appliesOn) {
    $appliesOnArray = [];

    foreach ($appliesOn as $value) {
      list($entityTypeId, $bundle) = explode('.', $value . '.');
      if (!isset($appliesOnArray[$entityTypeId])) {
        $appliesOnArray[$entityTypeId] = [];
      }
      if (!empty($bundle)) {
        $appliesOnArray[$entityTypeId][] = $bundle;
      }
    }

    return $appliesOnArray;
  }

}
